#Eneroth Texture Positioning Tools

#Author:	Julia Christina Eneroth, Eneroth3, eneroth3@gmail.com

module Ene_texturePositioning

#Menu
menu = UI.menu("Plugins").add_submenu("Texture Positioning")
menu.add_item("Align Texture") { self.align_material }
menu.add_item("Align Texture To Edge") { self.align_material(true) }
menu.add_item("Rotate Counter Clockwise"){ self.rotateTexture(90) }
menu.add_item("Rotate Clockwise"){ self.rotateTexture(-90) }
menu.add_item("Rotate Custom angle"){ self.rotateTexture }
menu.add_item("Depatternize"){ self.depatternize }

#Toolbar
tb = UI::Toolbar.new "Texture Positioning"

cmd = UI::Command.new("Align Texture") { self.align_material }
cmd.large_icon = "align_undefinedEdge.png"
cmd.small_icon = "align_undefinedEdge_small.png"
cmd.tooltip = "Align Texture"
cmd.status_bar_text = "Align texture(s) to individual face's side."
tb.add_item cmd

cmd = UI::Command.new("Align Texture To Edge") { self.align_material(true) }
cmd.large_icon = "align_definedEdge.png"
cmd.small_icon = "align_definedEdge_small.png"
cmd.tooltip = "Align Texture To Edge"
cmd.status_bar_text = "Align Texture(s) to selected edge."
tb.add_item cmd

cmd = UI::Command.new("Rotate Counter Clockwise") { self.rotateTexture(90) }
cmd.large_icon = "rotate_cc.png"
cmd.small_icon = "rotate_cc_small.png"
cmd.tooltip = "Rotate Counter Clockwise"
cmd.status_bar_text = "Rotate textures(s) 90° counter clockwise."
tb.add_item cmd

cmd = UI::Command.new("Rotate Clockwise") { self.rotateTexture(-90) }
cmd.large_icon = "rotate_cw.png"
cmd.small_icon = "rotate_cw_small.png"
cmd.tooltip = "Rotate Clockwise"
cmd.status_bar_text = "Rotate textures(s) 90° clockwise."
tb.add_item cmd

cmd = UI::Command.new("Rotate Custom angle") { self.rotateTexture }
cmd.large_icon = "rotate_custom.png"
cmd.small_icon = "rotate_custom_small.png"
cmd.tooltip = "Rotate Custom angle"
cmd.status_bar_text = "Rotates the material of all selected faces counter clockwise a promped angle."
tb.add_item cmd

cmd = UI::Command.new("Depatternize") { self.depatternize }
cmd.large_icon = "depatternize.png"
cmd.small_icon = "depatternize_small.png"
cmd.tooltip = "Depatternize"
cmd.status_bar_text = "Move around textures randomly to prevent patterns between faces."
tb.add_item cmd

UI.start_timer(0.1, false){ tb.restore }#Use timer as workaround for bug 2902434.

#Align Texture
def self.align_material(definedEdge = false)
	#When definedEdge is false all face's textures are aligned to their own first bounding edge.
	#When true all face's textures are aligned to the one selected edge. Only one edge can then be selected
	
	ss = Sketchup.active_model.selection
	
	if definedEdge then									#Align to predefined (selected) edge

		ssEdges = []									# Put selected edges in there own array
		ss.each do |i|
			next unless i.class == Sketchup::Edge
			ssEdges << i
		end#each
		unless ssEdges[0]								# At least one edge must be selected
			UI.messagebox("An edge to align to must be selected.")
			return
		end#if
		if ssEdges[1] then								# More than one edge can't be selected
			UI.messagebox("Can only align to one edge. Select one and retry.")
			ss.toggle ssEdges							# Deselect all edges so user can select one (just one) and retry
			return
		end#if
		globalvector = ssEdges[0].line[1]						# Edge to align all faces too
		
	end#if
	
	Sketchup.active_model.start_operation "Align Texture"

	ss.each do |i|										#Loop all selected entities
	
		next unless i.class == Sketchup::Face			# Skip if not face
		next unless i.material							# Skip if no material or no texture
		next unless i.material.texture
		vector = i.edges[0].line[1] unless globalvector	# Align to first bounding edge if no edge is defined
		vector = globalvector if globalvector			# Otherwise use defined edge's vector
		next unless i.normal.perpendicular? vector		# Skip if vector isn't in the plane of the face
		achorPoint = i.edges[0].line[0]					# Define point to rotate around
			
		textureWidth = i.material.texture.width
		vector.length = textureWidth					# Change vector's length to materials width
		
		points = [achorPoint, [0,0,0], [achorPoint[0]+vector[0],achorPoint[1]+vector[1],achorPoint[2]+vector[2]], [1,0,0]]	# Reposition material
		i.position_material(i.material, points, true)
		
	end#each
	
	Sketchup.active_model.commit_operation

end#def

#Rotate Texture
def self.rotateTexture(angle=nil)
	#Prompt angle if not set
	
	unless angle											#Prompt angle if not defined
		input = UI.inputbox(["Angle (degrees, cc)"], [0], "Rotate Textures")
		return unless input
		angle = input[0]
	end#unless
	return unless angle
	
	Sketchup.active_model.start_operation "Rotate Texture"

	ss = Sketchup.active_model.selection
	
	ss.each do |i|											#Loop all selected entities
	
		next unless i.class == Sketchup::Face				# Skip if not face
		next unless i.material								# Skip if no material or no texture
		next unless i.material.texture
		
		tw = Sketchup.create_texture_writer					# Create uv helper to get current material position
		uvh = i.get_UVHelper true, false, tw
		
		trans = Geom::Transformation.rotation i.outer_loop.vertices[0].position, i.normal, angle * Math::PI / 180	#Define rotation
		
		pointPairs = []
		(0..1).each do |j|									# Loop some points around face
			point3d = i.outer_loop.vertices[j].position		#  Selet a 3d point
			point3dRotated = point3d.transform(trans)		#  Rotate 3d pont
			pointPairs << point3dRotated					#  Save model's 3d point to array
			point2d = uvh.get_front_UVQ(point3d)
			pointPairs << point2d							#  Save material's corresponding 2d point to array
		end#each

		i.position_material(i.material, pointPairs, true)	#Set material position (pair up model 3d points with texture 2d point)
		
	end#each

	Sketchup.active_model.commit_operation
	
end#def

#Texture Depatternize
def self.depatternize

	Sketchup.active_model.start_operation "Depatternize"

	ss = Sketchup.active_model.selection
	
	ss.each do |i|											#Loop all selected entities
	
		next unless i.class == Sketchup::Face				# Skip if not face
		next unless i.material								# Skip if no material or no texture
		next unless i.material.texture
		
		tw = Sketchup.create_texture_writer					# Create uv helper to get current material position
		uvh = i.get_UVHelper true, false, tw
		
		vector = i.edges[0].line[1]							# Get a vector in the face's plane
		vector.length = rand(i.material.texture.height+i.material.texture.width)							# Set random length (between 0 and materials height + width)
		trans = Geom::Transformation.rotation i.outer_loop.vertices[0].position, i.normal, rand(Math::PI*2)	# Rotate vector randomly in plane
		vector.transform! trans
		trans = Geom::Transformation.translation vector		# Create translation, move point by vector
		
		pointPairs = []
		(0..1).each do |j|									# Loop some points around face
			point3d = i.outer_loop.vertices[j].position		#  Select a 3d point
			point3dRotated = point3d.transform(trans)		#  Move 3d point
			pointPairs << point3dRotated					#  Save model's 3d point to array
			point2d = uvh.get_front_UVQ(point3d)
			pointPairs << point2d							#  Save material's corresponding 2d point to array
		end#each

		i.position_material(i.material, pointPairs, true)	#Set material position (pair up model 3d points with texture 2d point)
		
	end#each
	
	Sketchup.active_model.commit_operation

end#def

end#module